//------------------------------------------------------------------
//
// --- TIL BRUK I OPPGAVE 14-2 ---
//
// dato.cpp - implementasjonsfil    E.L. 2005-01-02
//              tilpasset lreboka 2006-04-03
//
#include <string>
#include <cctype>
#include <ctime>
#include "dato.h"

using namespace std;

const int antDagerIAaret = 365;
const int antDagerISkuddaar = antDagerIAaret + 1;
const int datoLengde = 8;

// Lengden av mnedene er lagt inn som navnlse konstanter i
// funksjonen mndDager().

//-------------------------------------------------------------------
//
// Prototyper for funksjoner som benyttes ved implementasjon av
// klassen Dato. Funksjonene er definert bakerst i denne filen.
//
bool datoOk(int dag, int mnd, int aar);
bool langMnd(int mnd);
int  mndDager(int mnd, int aar);
bool skuddaar(int aar);

//-------------------------------------------------------------------
//
// Implementasjon av klassen Dato
//
Dato::Dato()
{
  settDato();
} // Dato

//-------------------------------------------------------------------
Dato::Dato(int nyDag, int nyMnd, int nyttAar)
{
  settDato(nyDag, nyMnd, nyttAar);
} // Dato

//-------------------------------------------------------------------
Dato::Dato(const string &nyDato)
{
  settDato(nyDato);
} // Dato

//-------------------------------------------------------------------
bool Dato::erOk() const
{
  return ok;
} // erOk

//-------------------------------------------------------------------
int Dato::finnDag() const
{
  return dag;
} // finnDag

//-------------------------------------------------------------------
int Dato::finnMnd() const
{
  return mnd;
} // finnMnd

//-------------------------------------------------------------------
int Dato::finnAar() const
{
  return aar;
} // finnAar

//-------------------------------------------------------------------
bool Dato::settDato(int nyDag, int nyMnd, int nyttAar)
{
  ok = true;
  if (!datoOk(nyDag, nyMnd, nyttAar)) {
    ok = false;
    }
  else {
    dag = nyDag;
    mnd = nyMnd;
    aar = nyttAar;
    }
  return ok;
} // settDato

//-------------------------------------------------------------------
bool Dato::settDato(const string &nyDato)
{
  ok = true;

  // Frst feilkontroll av forskjellig slag.

  // Lengden m vre (minst) lik datoLengde - 1.
  int lengde = nyDato.length();
  if (lengde < datoLengde - 1) ok = false;

  if (ok) {
    // nyDato m bare inneholde sifre:
    int I = 0;
    while (I < lengde && isdigit(nyDato[I])) I++;
    if (I < lengde) ok = false;  // FEIL. Ikke-siffer er funnet

    if (ok) {
      // Henter ut siffer 1 og 2 som dag, siffer 3 og 4 som mned og
      // siffer 5-8 som rstallet:
      int nyDag = (nyDato[0] - '0') * 10 + (nyDato[1] - '0');
      int nyMnd = (nyDato[2] - '0') * 10 + (nyDato[3] - '0');
      int nyttAar = (nyDato[4] - '0') * 1000 + (nyDato[5] - '0') * 100 +
              (nyDato[6] - '0') * 10 + (nyDato[7] - '0');

      // sjekker om gyldige tall i datoen
      if (!datoOk(nyDag, nyMnd, nyttAar)) {
        ok = false;  // FEIL
        }
      else {
        dag = nyDag;
        mnd = nyMnd;
        aar = nyttAar;
        }
      }
    }
  return ok;
} // settDato

//-------------------------------------------------------------------
void Dato::settDato()
{
  // Bruker funksjoner i <ctime> til  finne dagens dato
  time_t t;  // datatypen time_t representerer kalendertid
  time(&t);
  tm *tid = localtime(&t);  // datatypen tm er en struktur
  dag = tid->tm_mday;
  mnd = tid->tm_mon + 1;  // tm_mon gir verdi 0-11
  aar = tid->tm_year + 1900;  // tm_year gir r etter 1900
} // settDato

//-------------------------------------------------------------------
void Dato::finnDato(int &dagen, int &mnden, int &aaret) const
{
  dagen = dag;
  mnden = mnd;
  aaret = aar;
} // finnDato

//-------------------------------------------------------------------
string Dato::finnDato() const
{
  // Lager en streng med riktig lengde og fyller den med blanke, kunne
  // ogs fylt den med et hvilket som helst annet tegn.
  string datoen(datoLengde, ' ');
  datoen[0] = (char)(dag / 10 + '0');
  datoen[1] = (char)(dag % 10 + '0');
  datoen[2] = (char)(mnd / 10 + '0');
  datoen[3] = (char)(mnd % 10 + '0');
  datoen[4] = (char)(aar / 1000 + '0');
  int rest = aar % 1000;
  datoen[5] = (char)(rest / 100 + '0');
  rest %= 100;
  datoen[6] = (char)(rest / 10 + '0');
  datoen[7] = (char)(rest % 10 + '0');
  return datoen;
} // finnDato

//-------------------------------------------------------------------
int Dato:: sammenlikn(const Dato &denAndre) const
{
  int like;
  if (aar < denAndre.aar) like = -1;
  else if (aar > denAndre.aar) like = +1;
  else { // rstallene er like
    int mndDag1 = mndDager(mnd, aar) + dag;
    int mndDag2 = mndDager(denAndre.mnd, denAndre.aar) + denAndre.dag;
    if (mndDag1 < mndDag2) like = -1;
    else if (mndDag1 > mndDag2) like = +1;
    else like = 0;
    }
  return like;
}  // sammenlikn

//-------------------------------------------------------------------
long int Dato::antallDagerTil(const Dato &denAndre) const
{
  if (sammenlikn(denAndre) == 0) return 0;    // RETUR. Like datoer
  if (aar == denAndre.aar) {
    return (mndDager(denAndre.mnd, denAndre.aar) + denAndre.dag -
            (mndDager(mnd, aar) + dag));      // RETUR. Samme r
    }

  // Forskjellig r, ordner datoene i "riktig" rekkeflge
  Dato t1;
  Dato t2;

  if (sammenlikn(denAndre) > 0) {
    t1 = denAndre;  // datamedlemmene blir kopiert over
    t2.dag = dag;
    t2.mnd = mnd;
    t2.aar = aar;
    }
  else {
    t2 = denAndre; // datamedlemmene blir kopiert over
    t1.dag = dag;
    t1.mnd = mnd;
    t1.aar = aar;
    }

  // antall hele r
  long int antDager = 0;
  int      aarNr;
  for (aarNr = t1.aar + 1; aarNr <= t2.aar - 1; aarNr++) {
    if (skuddaar(aarNr)) antDager += antDagerISkuddaar;
    else antDager += antDagerIAaret;
    }
  // antall dager i det frste ret
  if (skuddaar(t1.aar)) {
    antDager += antDagerISkuddaar - mndDager(t1.mnd, t1.aar) - t1.dag;
    }
  else {
    antDager += antDagerIAaret - mndDager(t1.mnd, t1.aar) - t1.dag;
    }
  // antall dager i det siste ret
  antDager += mndDager(t2.mnd, t2.aar) + t2.dag;

  if (sammenlikn(denAndre) > 0) return -antDager;
  else return antDager;
} // antallDagerTil

//-------------------------------------------------------------------
//
// Funksjonen returnerer true hvis ok dato
//
bool datoOk(
       int dag,   // Inn
       int mnd,   // Inn
       int aar)   // Inn
{
  bool ok = true;
  if (aar < 0) ok = false;
  if (mnd < 1 || mnd > 12) ok = false;
  if (dag < 1) ok = false;
  if (mnd == 2) {
   if (skuddaar(aar)) {
     if (dag > 29) ok = false;
     }
   else if (dag > 28) ok = false;
   }
  if (langMnd(mnd)) {
    if(dag > 31) ok = false;
    }
  else {  // kort mned (februar allerede dekket)
    if (dag > 30) ok = false;
    }
  return ok;
} // datoOk

//-------------------------------------------------------------------
//
// Funksjonen returnerer true hvis mnd har 31 dager
//
bool langMnd(
       int mnd)   // Inn
{
  if (mnd == 1 || mnd == 3 || mnd == 5 || mnd == 7 || mnd == 8 ||
      mnd == 10 || mnd == 12) return true;
  else return false;
} // langMnd

//-------------------------------------------------------------------
//
// Antall dager fram til (men ikke inkl.) mnd
//
int mndDager(
      int mnd,    // Inn
      int aar)    // Inn
{
  int antall = 0;
  if (mnd >  1) antall += 31;   // jan
  if (mnd >  2) {               // feb
    if (skuddaar(aar)) antall += 29;
    else antall += 28;
    }
  if (mnd >  3) antall += 31;   // mars
  if (mnd >  4) antall += 30;   // april
  if (mnd >  5) antall += 31;   // mai
  if (mnd >  6) antall += 30;   // juni
  if (mnd >  7) antall += 31;   // juli
  if (mnd >  8) antall += 31;   // aug
  if (mnd >  9) antall += 30;   // sept
  if (mnd > 10) antall += 31;   // okt
  if (mnd > 11) antall += 30;   // nov
  if (mnd > 12) antall += 31;   // des
  return antall;
} // mndDager

//-------------------------------------------------------------------
//
// Funksjonen returnerer true dersom ret er et skuddr
//
bool skuddaar(
       int aar)  // Inn
{
  if (aar % 400 == 0 || aar % 4 == 0 && aar % 100 != 0) return true;
  else return false;
} // skuddaar
