//-------------------------------------------------------------------
//
// prosjektoppgave.cpp - implementasjonsfil
//
// Implementasjon av klassene Student og Prosjektoppgave
//
#include <cctype>
#include "prosjektoppgave.h"

//-------------------------------------------------------------------
Student::Student(const string &initNavn) 
       : navn(initNavn) 
{ 
  karakter = karakterIkkeOppgitt;  
} // Student

//-------------------------------------------------------------------
Student::Student(const Navn &initNavn) 
       : navn(initNavn) 
{ 
  karakter = karakterIkkeOppgitt;  
} // Student

//-------------------------------------------------------------------
const Navn &Student::finnNavn() const
{
  return navn;
} // finnNavn

//-------------------------------------------------------------------
char Student::finnKarakter() const
{
  return karakter;
} // finnKarakter

//-------------------------------------------------------------------
void Student::settNavn(const string &nyttNavn) 
{
  navn.settNavn(nyttNavn);
} // settNavn

//-------------------------------------------------------------------
void Student::settNavn(const Navn &nyttNavn) 
{
  navn = nyttNavn;
} // settNavn

//-------------------------------------------------------------------
bool Student::settKarakter(char nyKarakter) 
{
  nyKarakter = toupper(nyKarakter);
  if (nyKarakter >= nedreKarGrense && nyKarakter <= ovreKarGrense) {
    karakter = nyKarakter;
    return true;
  } else return false;
} // settKarakter

//-------------------------------------------------------------------
string Student::lagUtskrift() const
{
  ostringstream utskrift;
  utskrift << "Student, navn: " << navn.finnNavn();
  if (karakter != karakterIkkeOppgitt) {
    utskrift << ", karakter " << karakter;
    }
  else utskrift << ", karakter ikke satt.";
  return utskrift.str();
} // lagUtskrift

//-------------------------------------------------------------------
Prosjektoppgave::Prosjektoppgave(int initNr, const string &initTittel,
                   const string &initStudNavn, 
                   const string &initNavnVeileder)
               : oppgaveNr(initNr), 
                 tittel(initTittel), 
                 student(initStudNavn),
                 veileder(initNavnVeileder)
{ } // Prosjektoppgave

//-------------------------------------------------------------------
void Prosjektoppgave::settNavnStudent(const string &nyttNavn)
{
  student.settNavn(nyttNavn);
} // settNavnStudent

//-------------------------------------------------------------------
void Prosjektoppgave::settNavnVeileder(const string &nyttNavn)
{ 
  veileder.settNavn(nyttNavn);
} // settNavnVeileder

//-------------------------------------------------------------------
void Prosjektoppgave::settNavnStudent(const Navn &nyttNavn)
{
  student.settNavn(nyttNavn);
} // settNavnStudent

//-------------------------------------------------------------------
void Prosjektoppgave::settNavnVeileder(const Navn &nyttNavn)
{ 
  veileder = nyttNavn;
} // settNavnVeileder

//-------------------------------------------------------------------
bool Prosjektoppgave::settKarakter(char nyKarakter)
{
  return student.settKarakter(nyKarakter);
} // settKarakter

//-------------------------------------------------------------------
int Prosjektoppgave::finnNr() const
{
  return oppgaveNr;
} // finnNr

//-------------------------------------------------------------------
const string &Prosjektoppgave::finnTittel() const
{
  return tittel;
} // finnTittel

//-------------------------------------------------------------------
const Navn &Prosjektoppgave::finnStudnavn() const
{
  return student.finnNavn();
} // finnStudnavn

//-------------------------------------------------------------------
const Navn &Prosjektoppgave::finnVeiledernavn() const
{
  return veileder;
} // finnVeiledernavn

//-------------------------------------------------------------------
char Prosjektoppgave::finnKarakter() const
{
  return student.finnKarakter();
} // finnKarakter

//-------------------------------------------------------------------
string Prosjektoppgave::lagUtskrift() const
{
  ostringstream utskrift;
  utskrift << "Oppgave nr " << oppgaveNr << " " << tittel << "\n" <<
     student.lagUtskrift() << "\nVeileder: " << veileder.finnNavn();
  return utskrift.str();
} // lagUtskrift