//-------------------------------------------------------------------
//
// algoritmerEndre.cpp
//
// Eksempler p algoritmer som endrer eller flytter p datainnholdet:
//    swap(), copy(), swap_ranges(), fill(), resize(),
//    replace_copy(), reverse_copy(), rotate(), next_permutation()
//
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
using namespace  std;

vector<int> lagVektor1();
vector<int> lagVektor2();
void        skrivUt(const string &tekst, const vector<int> &vektoren);

int main()
{
  vector<int> v1 = lagVektor1();
  vector<int> v2 = lagVektor2();
  skrivUt("Vektor 1: " , v1);
  skrivUt("Vektor 2: " , v2);

  swap(v1, v2); // bytter om v1 og v2
  cout << "Etter ombytting:\n";
  skrivUt("Vektor 1: " , v1);
  skrivUt("Vektor 2: " , v2);

  int gmlStr = v1.size();
  v1.resize(v1.size() + v2.size());
  copy(v2.begin(), v2.end(), v1.begin() + gmlStr);
  skrivUt("N er v1 lik v1 + v2: " , v1);

  // Bytter om elementene [0, 5> og [5, 10>
  swap_ranges(v1.begin(), v1.begin() + 5, v1.begin() + 5);
  skrivUt("V1, byttet om elementene [0, 5> og [5, 10>: " , v1);

  // Legger inn 3 i de fem frste elementene
  fill(v1.begin(), v1.begin() + 5, 3);
  skrivUt("V1 med tallet 3 i element [0, 5>: " , v1);

  // Lager en ny vektor der alle 3-tall er byttet ut med 300
  vector<int> kopi;
  kopi.resize(v1.size());
  replace_copy(v1.begin(), v1.end(), kopi.begin(), 3, 300);
  skrivUt("Kopi: ", kopi);

  // Snur v1
  vector<int> snuddV1(v1.size());
  reverse_copy(v1.begin(), v1.end(), snuddV1.begin());
  skrivUt("V1, snudd: ", snuddV1);

  // Roterer v1 3 posisjoner
  skrivUt("V1, fr rotasjonen: ", v1);
  rotate(v1.begin(), v1.begin() + 3, v1.end());
  skrivUt("V1, rotert 3 posisjoner: ", v1);

  // Lager en vektor med strrelse 3, finner deretter alle permutasjoner
  vector<int> ny;
  ny.push_back(1);
  ny.push_back(2);
  ny.push_back(3);
  // Antall permutasjoner er lik 3! = 1*2*3 = 6.
  // Lager sju permutasjoner, da skal perm. 7 vre lik perm. 1
  for (int i = 0; i < 7; i++) {
    next_permutation(ny.begin(), ny.end());
    cout << "Permutasjon " << (i + 1) << ":";
    skrivUt("", ny);
  }
  return 0;
} // main

//-------------------------------------------------------------------
//
// Funksjonene lager to tallvektorer med utgangspunkt i tabeller.
// For eksemplets skyld vises to mter  gjre dette p.
//
vector<int> lagVektor1()
{
  const int antall = 7;
  int tabell[antall] = {3, 3, 12, 14, 17, 25, 30};

  // Bruker en spesialkonstruktr
  vector<int> v1(tabell, tabell + antall);

  return v1;
} // lagVektor1

//-------------------------------------------------------------------
vector<int> lagVektor2()
{
  const int antall = 5;
  int tabell[antall] = {2, 3, 12, 14, 24};

  // Bruker assign direkte
  vector<int> v2;
  v2.assign(tabell, tabell + antall);
  return v2;
} // lagVektor2

//-------------------------------------------------------------------
//
// Funksjonen skriver ut datainnholdet i en vektor.
//

void skrivUt(
  const string      &tekst,     // Inn
  const vector<int> &vektoren)  // Inn
{
  cout << tekst << endl;
  for (int i = 0; i < vektoren.size(); i++) {
    cout << vektoren[i] << " ";
  }
  cout << endl;
} // skrivUt
