//-------------------------------------------------------------------
//
// algoritmerPaaSorterteData.cpp
//
// Viser eksempel p bruk av flgende algoritmer som forutsetter
//    sorterte data:  binary_search(), includes(),
//    upper_bound(), merge(), set_union(), set_intersection()
// Viser ogs eksempler p bruk av
//    resize(), insert(), unique() og erase()
//
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
using namespace  std;

vector<int> lagVektor1();
vector<int> lagVektor2();
void        skrivUt(const string &tekst, const vector<int> &vektoren);

int main()
{
  vector<int> v1 = lagVektor1();
  vector<int> v2 = lagVektor2();
  skrivUt("Vektor 1: " , v1);
  skrivUt("Vektor 2: " , v2);

  // Binrsk. Funksjonen binary_search() sier om et element
  // er i en vektor, men ikke hvor. Bruker lower_bound()
  // (ev. upper_bound()) til  fortelle oss det.
  if (binary_search(v1.begin(), v1.end(), 17)) {
    cout << "17 fins i v1\n";
    }
  else cout << "17 fins ikke i v1\n";

  // Verdien 17 er i v1, men p hvilken indeks?
  vector<int>::iterator pos = lower_bound(v1.begin(), v1.end(), 17);
  cout << "17 er p indeks " << (pos - v1.begin()) << endl;

  // 25 er ikke i v2. Lager en kopi av v2, skal sette inn 25 der
  vector<int> v3 = v2;
  pos = lower_bound(v3.begin(), v3.end(), 25);
  cout << "25 skal inn p posisjon " << (pos - v3.begin()) << endl;
  v3.insert(pos, 25);
  skrivUt("V3 er lik v2 med 25 satt inn:", v3);

  // Includes
  if (includes(v1.begin(), v1.end(),
      v2.begin() + 1, v2.begin() + 3)) cout << "Er med!\n";
  else cout << "Er ikke med!\n";

  // Fletting
  vector <int> resultat;
  resultat.resize(v1.size() + v2.size()); // eksakt riktig strrelse
  merge(v1.begin(), v1.end(), v2.begin(), v2.end(), resultat.begin());
  skrivUt("V1 og V2 flettet:", resultat);

  // Fjerner dubletter i resultat
  vector<int>::iterator ende = unique(resultat.begin(), resultat.end());
  resultat.erase(ende, resultat.end());
  skrivUt("V1 og V2 flettet, uten dubletter:", resultat);

  // Mengdeoperasjoner
  resultat.resize(v1.size() + v2.size()); // minst stort nok
  ende = set_union(v1.begin(), v1.end(), v2.begin(), v2.end(),
                                          resultat.begin());
  resultat.erase(ende, resultat.end()); // reduserer strrelsen
  skrivUt("V1 union V2:", resultat);

  resultat.resize(v1.size() + v2.size()); // minst stort nok
  ende = set_intersection(v1.begin(), v1.end(), v2.begin(), v2.end(),
                                                resultat.begin());
  resultat.erase(ende, resultat.end());
  skrivUt("V1 snitt V2:", resultat);

  return 0;
} // main

//-------------------------------------------------------------------
//
// Funksjonene lager to tallvektorer med utgangspunkt i tabeller.
// For eksemplets skyld vises to mter  gjre dette p.
//
vector<int> lagVektor1()
{
  const int antall = 7;
  int tabell[antall] = {3, 3, 12, 14, 17, 25, 30};

  // Bruker en spesialkonstruktr
  vector<int> v1(tabell, tabell + antall);

  return v1;
} // lagVektor1

//-------------------------------------------------------------------
vector<int> lagVektor2()
{
  const int antall = 5;
  int tabell[antall] = {2, 3, 12, 14, 24};

  // Bruker assign direkte
  vector<int> v2;
  v2.assign(tabell, tabell + antall);
  return v2;
} // lagVektor2

//-------------------------------------------------------------------
//
// Funksjonen skriver ut datainnholdet i en vektor.
//

void skrivUt(
  const string      &tekst,     // Inn
  const vector<int> &vektoren)  // Inn
{
  cout << tekst << endl;
  for (int i = 0; i < vektoren.size(); i++) {
    cout << vektoren[i] << " ";
  }
  cout << endl;
} // skrivUt
