//-------------------------------------------------------------------
//
// algoritmerSoek.cpp
//
// Viser eksempel p bruk av flgende algoritmer:
//    find(), find_first_of(), adjacent_find(), count() og
//    max_element()
//
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
using namespace  std;

vector<int> lagVektor1();
vector<int> lagVektor2();
void        skrivUt(const string &tekst, const vector<int> &vektoren);

int main()
{
  vector<int> v1 = lagVektor1();
  vector<int> v2 = lagVektor2();
  skrivUt("Vektor 1: " , v1);
  skrivUt("Vektor 2: " , v2);

  // Sker etter enkeltverdi med find()
  vector<int>::const_iterator pos = find(v1.begin(), v1.end(), 25);
  if (pos != v1.end()) {
    cout << "25 fins i v1 p indeks " << (pos - v1.begin()) << endl;
  } else cout << "25 fins ikke i v1\n";

  vector<int>::const_iterator start = v1.begin() + 2;
  vector<int>::const_iterator slutt = v1.end() - 2;
  vector<int>::const_iterator it;
  cout << "Sker i en del av tabellen: ";
  for (it = start; it != slutt; it++) cout << *it << " ";
  cout << endl;
  pos = find(start, slutt, 14);
  if (pos != slutt) {
    cout << "14 fins i v1 p indeks " << (pos - v1.begin()) << endl;
  } else cout << "14 fins ikke i v1\n";

  // Sker etter den frste blant flere, find_first_of()
  // Sker i v1 {3, 3, 12, 14, 17, 25, 30} etter den frste av
  // {12, 14, 24} (en del av v2). Svaret skal bli 2
  pos = find_first_of(v1.begin(), v1.end(), v2.begin() + 2, v2.end());
  cout << "Frste p indeks " << (pos - v1.begin()) << endl;

  // Sker etter to like, ved siden av hverandre, adjacent_find()
  pos = adjacent_find(v1.begin(), v1.end());
  cout << "Finner to like p indeks " << (pos - v1.begin()) << endl;

  // Teller antall 3-tall i v1
  cout << "v1 inneholder " << count(v1.begin(), v1.end(), 3)
                           << " 3-tall\n";

  // Finner strste verdi i v1 (max_element returnerer en iterator)
  cout << "Strste verdi i v1 er " <<
        *max_element(v1.begin(), v1.end()) << endl; // dereferanse

  return 0;
} // main

//-------------------------------------------------------------------
//
// Funksjonene lager to tallvektorer med utgangspunkt i tabeller.
// For eksemplets skyld vises to mter  gjre dette p.
//
vector<int> lagVektor1()
{
  const int antall = 7;
  int tabell[antall] = {3, 3, 12, 14, 17, 25, 30};

  // Bruker en spesialkonstruktr
  vector<int> v1(tabell, tabell + antall);

  return v1;
} // lagVektor1

//-------------------------------------------------------------------
vector<int> lagVektor2()
{
  const int antall = 5;
  int tabell[antall] = {2, 3, 12, 14, 24};

  // Bruker assign direkte
  vector<int> v2;
  v2.assign(tabell, tabell + antall);
  return v2;
} // lagVektor2

//-------------------------------------------------------------------
//
// Funksjonen skriver ut datainnholdet i en vektor.
//

void skrivUt(
  const string      &tekst,     // Inn
  const vector<int> &vektoren)  // Inn
{
  cout << tekst << endl;
  for (int i = 0; i < vektoren.size(); i++) {
    cout << vektoren[i] << " ";
  }
  cout << endl;
} // skrivUt
